#!/usr/bin/env python

import paramiko
import time
from multiprocessing import Pool
from juraSSH import file_handler as fh

"""This module will be used to send multiple commands multiple remote machines in parallel.
     """


USERNAME = None
PASSWORD = None

HOSTS_FILE = []
COMMANDS = []

PORT = 22
KEY = False

TIMEOUT = 10
SLEEP_TIME = 8


def connector(hostname):
    """This function allows you to connect to multiple remote machines
    """

    ssh = paramiko.SSHClient()

    try:
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=hostname, username=USERNAME,
                    password=PASSWORD, look_for_keys=KEY, timeout=TIMEOUT, port=PORT)
        print("Connected to {0}\n".format(hostname))

    except:
        fh.file_writer(hostname)  # If a host is unreachable, it will be recorded in a file.
        ssh.close()
    else:
        channel = ssh.invoke_shell()
        channel_data = str()

        for cmd in COMMANDS:

            channel.send(cmd + "\n")
            time.sleep(4)

            output = channel.recv(99999)
            time.sleep(SLEEP_TIME)

            channel_data += output
            channel_data += "\n" * 4

        print(channel_data)
        print("#" * 120)
        print("\n")

        channel.close()
        ssh.close()
        return channel_data


def run_multiple_cmds(username, password, hosts_file, commands_file, cores=25):
    """This function will be used to send multiple commands to remote machines in parallel.
     """
    global USERNAME
    global PASSWORD
    global HOSTS_FILE
    global COMMANDS

    USERNAME = username
    PASSWORD = password
    HOSTS_FILE = hosts_file
    COMMANDS = commands_file

    started = time.asctime()

    p = Pool(cores)
    result = p.map(connector, HOSTS_FILE)
    p.close()
    p.join()

    print("\nProcessing completed !!\n".upper())
    print("Started at: {0}".format(started))
    print("Ended   at: {0}".format(time.asctime()))
    print("\nPLEASE RUN THE [ls] COMMAND TO SEE THE FILE WITH ALL UNREACHABLE HOSTS\n")

















#!/usr/bin/env python

import paramiko
import time
from multiprocessing import Pool
from juraSSH import file_handler as fh

__author__ = "David Johnnes"
__email__ = "david.johnnes@gmail.com"

"""
    This module allows you to send a single command to multiple remote host in parallel
"""

USERNAME = ""
PASSWORD = ""

HOSTS_FILE = []
COMMAND = ""

PORT = 22
KEY = False

TIMEOUT = 10
SLEEP_TIME = 8


def connector(hostname):
    """This function allows you to connect to multiple remote machines
    """
    ssh = paramiko.SSHClient()
    try:
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        ssh.connect(hostname=hostname, username=USERNAME, password=PASSWORD,
                    port=PORT, look_for_keys=KEY, timeout=TIMEOUT)
        print("\nConnected to == >> {0}\n".format(hostname))
    except:
        fh.file_writer(hostname)    # If a host is unreachable, it will be recorded in a file.
        ssh.close()
    else:
        channel = ssh.invoke_shell()
        channel.send(COMMAND)
        channel.send("\n")
        time.sleep(4)

        output = channel.recv(9999)
        time.sleep(SLEEP_TIME)

        print(output)
        print("#" * 120)
        channel.close()
        ssh.close()


def run_one_cmd(username, password, hosts_file, command, cores=25):
    """This function will be used to send single command to multiple remote machines in parallel.
     """
    global USERNAME
    global PASSWORD
    global HOSTS_FILE
    global COMMAND

    USERNAME = username
    PASSWORD = password
    HOSTS_FILE = hosts_file
    COMMAND = command

    started = time.asctime()

    p = Pool(cores)
    result = p.map(connector, HOSTS_FILE)
    p.close()
    p.join()

    print("\nProcessing completed !!\n".upper())
    print("Started at: {0}".format(started))
    print("Ended   at: {0}".format(time.asctime()))
    print("\nPLEASE RUN THE [ls] COMMAND TO SEE THE FILE WITH ALL UNREACHABLE HOSTS\n")
















from time import gmtime, strftime


def file_writer(data):
    """This function will be use to append all unreachable hosts to the 'unreachable_hosts file'
       Each file will have the timestamp appended to it, to make it easy to track.
    """

    with open("unreachable-hosts.txt", mode="a") as f:
        f.write(data)
        f.write("\n")

